Imports System.Net
Imports System.Net.Sockets
Imports System.IO.Pipes

''' <summary>
''' Dieses Form kann sowohl als Server-Anwendung laufen, als auch als Client-Anwendung
''' </summary>
Public Class frmMain

   Private _PipeName As String = "HalloWelt"
   Private WithEvents _ServerOrClient As CommunicationBase 'polymorphes Objekt

   Public Sub New()
      InitializeComponent()
      With Application.OpenForms 'bei mehreren Instanzen versetzt anordnen
         If .Count > 0 Then
            Me.Location = .Item(.Count - 1).Location + New Size(30, 50)
         Else
            Me.Location = Screen.PrimaryScreen.WorkingArea.Location
         End If
      End With
   End Sub

   Private Sub MenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) _
           Handles _
           SendMessageToolStripMenuItem.Click, _
           AddFormToolStripMenuItem.Click, _
           BeServerToolStripMenuItem.Click, _
           BeClientToolStripMenuItem.Click, _
           TestToolStripMenuItem.Click
      Select Case True
         Case sender Is SendMessageToolStripMenuItem
            If _ServerOrClient Is Nothing Then
               DisplayStatusLine("Fehler:", Lf, "no Connection")
            Else
               _ServerOrClient.Send(Me.rtbSend.Text)
            End If

         Case sender Is AddFormToolStripMenuItem
            Call (New frmMain()).Show(Me)

         Case sender Is BeServerToolStripMenuItem
            DisposeAll(_ServerOrClient)
            _ServerOrClient = New Server(_PipeName)
            DisplayStatusLine("Hi!", Lf, "I'm the Server, listening to:", Lf, """", _PipeName, """")
            Me.Text = "Server"

         Case sender Is BeClientToolStripMenuItem
            DisposeAll(_ServerOrClient)
            Dim clnt = New NamedPipeClientStream(".", _PipeName, PipeDirection.InOut, PipeOptions.Asynchronous)
            clnt.Connect()
            _ServerOrClient = New Client(clnt)
            DisplayStatusLine("Hi!", Lf, "I'm a Client, connected with:", Lf, """", _PipeName, """")
            Me.Text = "Client"

         Case sender Is TestToolStripMenuItem

      End Select
   End Sub

   'diese Sub ist normalerweise in der Designer.vb. Ich will aber sicherstellen kann, da _ServerOrClient **vor** den components disposed. Sonst empfngter womglich noch Daten, und will die darstellen.
   Protected Overrides Sub Dispose(ByVal disposing As Boolean)
      DisposeAll(_ServerOrClient)
      If disposing AndAlso components IsNot Nothing Then
         components.Dispose()
      End If
      MyBase.Dispose(disposing)
   End Sub

#Region "ServerOrClient-Ereignisverarbeitung"

   Private Sub ServerOrClient_ChatMessage( _
           ByVal Sender As Object, _
           ByVal e As MessageEventargs) Handles _ServerOrClient.ChatMessage
      DisplayLine(e.Message)
   End Sub

   Private Sub _ServerOrClient_Disposed(ByVal Sender As CommunicationBase) Handles _ServerOrClient.Disposed
      _ServerOrClient = Nothing
      Me.Text = "Offline"
   End Sub

   Private Sub ServerOrClient_StatusMessage( _
           ByVal Sender As Object, _
           ByVal e As MessageEventargs) Handles _ServerOrClient.StatusMessage
      DisplayStatusLine(e.Message)
   End Sub

#End Region 'ServerOrClient-Ereignisverarbeitung

   Private Sub DisplayLine(ByVal Msg As String)
      Me.rtbLog.AppendText(Msg & Lf)
   End Sub

   ''' <summary>Meldung hellgelb hinterlegt anzeigen</summary>
   Private Sub DisplayStatusLine(ByVal ParamArray Items() As Object)
      Static StatusLineColor As Color = Color.FromArgb(255, 255, 200)
      Me.rtbLog.SelectionBackColor = StatusLineColor
      DisplayLine(String.Concat(Items))
      Me.rtbLog.SelectionBackColor = Color.White
   End Sub
End Class